# -*- coding: utf-8 -*-

from model.network.Node import Node
from model.network.Edge import Edge
from model.network.Graph import Graph
from view.ASubWindow import ASubWindow
from view.network.GraphWidget import GraphWidget
from view.network.TableModel import (
    GraphTableModel, ComboBoxDelegate, TrueFalseComboBoxDelegate,
)

from PyQt5.QtCore import (
    Qt, QRect, QVariant, QAbstractTableModel, pyqtSlot, pyqtSignal,
    QEvent,
)

from PyQt5.QtWidgets import (
    QTableView, QItemDelegate, QComboBox, QLineEdit, QHBoxLayout, QSlider,
    QPushButton, QCheckBox, QStyledItemDelegate, QStyleOptionButton, QStyle,
    QApplication,
)

class NetworkWindow(ASubWindow):
    def __init__(self, model=None, title="River network", parent=None):
        super(NetworkWindow, self).__init__(name=title, ui="Network", parent=parent)
        self.ui.setWindowTitle(title)

        self.model = model
        if not self.model.data["graph"]:
            self.graph = Graph()
            self.model.data["graph"] = self.graph
        else:
            self.graph = self.model.data["graph"]

        # Graph Widget

        self.graph_widget = GraphWidget(self.graph, parent=self)
        self.graph_layout = self.find(QHBoxLayout, "horizontalLayout_graph")
        self.graph_layout.addWidget(self.graph_widget)

        # Nodes table

        self.nodes_model = GraphTableModel(
            headers = ["name", "type"],
            graph = self.graph,
            rows_type="nodes",
        )
        table = self.find(QTableView, "tableView_nodes")
        table.setModel(self.nodes_model)
        #table.resizeColumnsToContents()

        # Edges table

        self.reachs_model = GraphTableModel(
            headers = ["name", "enable", "node1", "node2"],
            graph = self.graph,
            rows_type="edges"
        )
        self.delegate_combobox = ComboBoxDelegate(
            graph = self.graph,
            parent = self,
        )
        self.delegate_true_false_combobox = TrueFalseComboBoxDelegate(
            parent = self,
        )

        table = self.find(QTableView, "tableView_reachs")
        table.setModel(self.reachs_model)
        table.setItemDelegateForColumn(1, self.delegate_true_false_combobox)
        table.setItemDelegateForColumn(2, self.delegate_combobox)
        table.setItemDelegateForColumn(3, self.delegate_combobox)
        #table.resizeColumnsToContents()

        # Connection

        self.nodes_model.dataChanged.connect(self.reachs_model.update)
        self.nodes_model.dataChanged.connect(self.graph_widget.rename_nodes)
        self.reachs_model.dataChanged.connect(self.graph_widget.display_update)
        self.reachs_model.dataChanged.connect(self.nodes_model.update)
        self.graph_widget.changeEdge.connect(self.reachs_model.update)
        self.graph_widget.changeNode.connect(self.nodes_model.update)

        self.find(QPushButton, "pushButton_add").clicked.connect(
            self.clicked_add
        )
        self.find(QPushButton, "pushButton_del").clicked.connect(
            self.clicked_del
        )
        self.find(QPushButton, "pushButton_reverse").clicked.connect(
            self.reverse_edge
        )

    def clicked_add(self):
        if self.get_push_button_checkable("pushButton_add"):
            self.set_push_button_checkable("pushButton_del", False)
            self.graph_widget.state("add")
        else:
            self.graph_widget.state("move")

    def clicked_del(self):
        if self.get_push_button_checkable("pushButton_del"):
            self.set_push_button_checkable("pushButton_add", False)
            self.graph_widget.state("del")
        else:
            self.graph_widget.state("move")

    def keyPressEvent(self, event):
        key = event.key()

        if key == Qt.Key_Escape:
            self.graph_widget.reset_selection

    def reverse_edge(self):
        """Reverse edge direction of selected table rows

        Returns:
            Nothing
        """
        indices = self.find(QTableView, "tableView_reachs").selectionModel().selectedIndexes()
        indexes = {}

        for index in sorted(indices):
            indexes[index.row()] = index

        for row in indexes:
            self.reachs_model.reverse_edge(indexes[row])
